function Memberships = GetFuzzyMemberships(Samples, Centers, NormType, Blending)
//////////////////////////////////////////////////////////////////////////// 
// CLUSTER - Clustering Toolbox
//
// Copyright (c) by Dr. Eng. (J) Harald Galda, 2011
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////


 // Input and output parameters are checked.
 
 [NumberOfOutputs NumberOfInputs] = argn();
 
 if (NumberOfInputs < 3) | (NumberOfInputs > 4) then
 
  error('Wrong number of inputs.');
  
 end;
 
 // Samples must be a matrix.
 
 if (typeof(Samples) ~= 'constant') | (ndims(Samples) ~= 2)  then
  
  error('Samples must be a 2D matrix.');
  
 end;
 
 // Centers must be a matrix.
 
 if (typeof(Centers) ~= 'constant') | (ndims(Centers) ~= 2)  then
  
  error('Centers must be a 2D matrix.');
  
 end;
 
 // Samples and Centers must have the same number of columns.
 
 if size(Samples, 2) ~= size(Centers, 2) then
  
  error('Samples and Centers must have the same number of columns');
  
 end;
 
 // NormType must be a scalar greater than zero.
 
 if (typeof(NormType) ~= 'constant') | (~isequal(size(NormType), [1 1])) then
 
  error('NormType must be a scalar greater than zero.');
 
 end;
 
 if NormType <= 0 then
  
  error('NormType must be a scalar greater than zero.');
  
 end;
 
 // Blending must be a scalar greater than one.
 
 if NumberOfInputs == 4 then

  if (typeof(Blending) ~= 'constant') | (~isequal(size(Blending), [1 1])) then
 
  error('Blending must be a scalar greater than one.');
 
  end;
 
  if Blending <= 1 then
  
   error('Blending must be a scalar greater than one.');
  
  end;
 
 else

  Blending = 2;
    
 end;

 if NumberOfOutputs ~= 1 then
  
  error('Wrong number of outputs.');
  
 end;
 
 // For each pair of sample and cluster center the inverse distance is calcula-
 // ted. 
 
 InverseDistances = CreateDistanceMatrix(Samples, Centers, NormType);
 
 InverseDistances(InverseDistances > 0) =  ...
 InverseDistances(InverseDistances > 0) .^ (2 / (1 - Blending));
 
 Memberships = zeros(size(InverseDistances, 1), size(InverseDistances, 2));
 
 // If a sample is identical to one or more cluster centers, the membership in
 // these clusters is 1 / (number of identical clusters) whereas the membership
 // in the other clusters is zero.
 
 for n = 1 : size(Samples, 1)

  Matches = find(InverseDistances(n, :) == 0);

  if isempty(Matches) then

   Memberships(n, :) = InverseDistances(n, :) / sum(InverseDistances(n, :));

  else
      
   Memberships(n, Matches) = Memberships(n, Matches) / length(Matches);

  end;
  
 end;

endfunction