function SOMWeights = InitSOM(Samples, GridSize, GridType)
//////////////////////////////////////////////////////////////////////////// 
// CLUSTER - Clustering Toolbox
//
// Copyright (c) by Dr. Eng. (J) Harald Galda, 2011
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////


 // Input and output parameters are checked.
 
 [NumberOfOutputs NumberOfInputs] = argn();
 
 if NumberOfInputs ~= 3 then

  error('Wrong number of inputs.');

 end;

 // Samples must be a matrix.
 
 if (typeof(Samples) ~= 'constant') | (ndims(Samples) ~= 2)  then
  
  error('Samples must be a 2D matrix.');
  
 end;

 // GridSize must be a vector or a scalar. If GridSize is a vector, all elements
 // must be integer numbers greater than one. The length of the vector must not
 // exceed the number of columns of Samples. 
 
 if (typeof(GridSize) ~= 'constant') | (ndims(GridSize) ~= 2) then

  error('GridSize must be an integer scalar or vector of integer numbers.');

 end;
 
 if isvector(GridSize) then

  if length(GridSize) > size(Samples, 2) then

   error('The length of GridSize must not exceed ' + ...
         'the number of columns of Samples');

  end;
 
  IllegalElements = find((GridSize <= 1) | (GridSize ~= round(GridSize)));
 
  if ~isempty(IllegalElements) then

   error('All elements of GridSize must be integer numbers greater than one.');

  end;

  if length(GridSize) > size(Samples, 2) then

   error('The number of elements of GridSize must not exceed the number ' + ...
         'of columns of Samples');

  end;
  
 else

  if ~isequal(size(GridSize), [1 1]) then

   error('GridSize must be an integer scalar or vector of integer numbers.');

  end;

  if (GridSize <= 1) | (GridSize ~= round(GridSize)) then

   error('If GridSize is a scalar, it must be an integer greater than one.');

  end;
  
 end;
 
 // GridType must be a string and can be 'equidistant' or 'adjusted' GridType
 // is not case sensitive.

 if (typeof(GridType) ~= 'string') | (~isequal(size(GridType), [1 1])) then
  
  error('GridType must be a string and can be ''equidistant'' or ' + ...
        '''adjusted''.');
 
 end;

 if NumberOfOutputs ~= 1 then

  error('Wrong number of outputs.');

 end;
 
 // The SOM weights are initialized.
 
 MeanOfSamples = mean(Samples, 'r');
 
 NumberOfNeurons = prod(GridSize);
 
 NumberOfDimensions = length(GridSize);
 
 SOMWeights = repmat(MeanOfSamples, NumberOfNeurons, 1);
 
 // A principal component analysis is applied to Samples. The principal axes 
 // corresponding to the biggest n eigenvalues are selected, with 
 // n == length(GridSize). For each principal component the absolute maximum 
 // value is determined. The range on the corresponding axis is the interval 
 // [-max(abs(Component)) max(abs(Component))].
 
 [Eigenvectors, PrincipalComponents, Eigenvalues, TSquare] = ...
 princomp(Samples, %t);
 
 SelectedIndices = 1 : NumberOfDimensions;
 
 // The coordinate system has the mean of samples as origin and the selected
 // eigenvectors as axes.
 
 AxisVectors = Eigenvectors(:, SelectedIndices)';
 
 // For each SOM dimension a list of points is generated.
 
 PointList = list();
 
 select convstr(GridType, 'l');

  case 'equidistant'
 
   for n = 1 : NumberOfDimensions

    MaxValue = norm(PrincipalComponents(:, SelectedIndices(n)), %inf);
    
    Step = 2 * MaxValue / (GridSize(n) - 1);
    
    PointList($ + 1) = -MaxValue : Step : MaxValue;
    
   end;
   
  case 'adjusted'
 
   NumberOfValues = size(PrincipalComponents, 1);
   
   for n = 1 : NumberOfDimensions

    // The current principal component is divided into intervals. Each interval
    // has approximately the same number of values.
    
    NumberOfIntervals = GridSize(n);
    
    NumberPerInterval = floor(NumberOfValues / NumberOfIntervals);

    SortedValues = gsort(PrincipalComponents(:, n));
    
    NumberOfRegularValues = NumberPerInterval * NumberOfIntervals;
    
    Intervals = matrix(SortedValues(1 : NumberOfRegularValues), ...
                       NumberPerInterval, ...
                       NumberOfIntervals);
    
    Points = zeros(1, NumberOfIntervals);
    
    if NumberOfValues == NumberOfRegularValues then
     
     Points = mean(Intervals, 'r');
    
    else
     
     Points(1 : $ - 1) = mean(Intervals(:, 1 : NumberOfIntervals - 1), 'r');
     
     LastInterval = cat(1, ...
                        Intervals(:, NumberOfIntervals), ...
                        SortedValues(NumberOfRegularValues + 1 : $));
                        
     Points($) = mean(LastInterval);
     
    end;
    
    PointList($ + 1) = Points;
    
   end;

  else    

   error('Unknown grid type.');
   
 end;

 // The SOM weights are calculated now by adding the weighted axis vectors
 // to the mean of the samples. Each axis vector is weighted by the list of
 // points of the corresponding SOM dimension.
 
 for n = 1 : NumberOfNeurons

  Indices = ind2sub(GridSize, n);

  for m = 1 : NumberOfDimensions
    
   Points = PointList(m);
     
   SOMWeights(n, :) = SOMWeights(n, :) ...
                    + (Points(Indices(m)) * AxisVectors(m, :));

  end;

 end;  

endfunction