function [Centers, Memberships] = FuzzyCMeans(Samples, ...
                                              InitialCenters, ...
                                              NormType, ...
                                              Criterion, ...
                                              Value, ...
                                              Blending)
//////////////////////////////////////////////////////////////////////////// 
// CLUSTER - Clustering Toolbox
//
// Copyright (c) by Dr. Eng. (J) Harald Galda, 2011
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

  // Input and output parameters are checked.
 
 [NumberOfOutputs NumberOfInputs] = argn();
 
 if (NumberOfInputs < 5) | (NumberOfInputs > 6) then
 
  error('Wrong number of inputs.');
  
 end;

 // Samples must be a matrix.
 
 if (typeof(Samples) ~= 'constant') | (ndims(Samples) ~= 2)  then
  
  error('Samples must be a 2D matrix.');
  
 end;
 
 // InitialCenters must be a matrix.
 
 if (typeof(InitialCenters) ~= 'constant') | (ndims(InitialCenters) ~= 2)  then
  
  error('InitialCenters must be a 2D matrix.');
  
 end;
 
 // Samples and InitialCenters must have the same number of columns.
 
 if size(Samples, 2) ~= size(InitialCenters, 2) then
  
  error('Samples and InitialCenters must have the same number of columns');
  
 end;
 
  // NormType must be a scalar greater than zero.
 
 if (typeof(NormType) ~= 'constant') | (~isequal(size(NormType), [1 1])) then
 
  error('NormType must be a scalar greater than zero.');
 
 end;
 
 if NormType <= 0 then
  
  error('NormType must be a scalar greater than zero.');
  
 end;
 
 // Criterion must be a string and can have the values 'iterations' or 
 // 'quantization_error'. Criterion is not case sensitive.
 
 if (typeof(Criterion) ~= 'string') | (~isequal(size(Criterion), [1 1])) then
  
  error('Criterion must be a string and can be ''iterations'' or ' + ...
        '''quantization_error''.');
 
 end;

 // Value must be a scalar greater than or equal to zero.
 
 if (typeof(Value) ~= 'constant') | (~isequal(size(Value), [1 1])) then
 
  error('Value must be a scalar greater than or equal to zero.');
 
 end;

 if Value < 0 then
  
  error('Value must be a scalar greater than or equal to zero.');
  
 end;
 
 // Blending must be a scalar greater than one.
 
 if NumberOfInputs == 6 then

  if (typeof(Blending) ~= 'constant') | (~isequal(size(Blending), [1 1])) then
 
  error('Blending must be a scalar greater than one.');
 
  end;
 
  if Blending <= 1 then
  
   error('Blending must be a scalar greater than one.');
  
  end;
 
 else

  Blending = 2;
    
 end; 
 
 if (NumberOfOutputs < 1) | (NumberOfOutputs > 2) then
  
  error('Wrong number of outputs.');
  
 end;
 
  // Initializations
 
 Centers = InitialCenters;

 NumberOfClusters = size(Centers, 1);

 IsCompleted = %f;
 
 // Main loop. The memberships of the row vectors of Samples in the clusters
 // given by the row vectors of centers are calculated. Then the termination
 // criterion is checked. If the termination criterion is fullfilled in the 
 // first iteration, Centers == InitialCenters.
 
 while ~IsCompleted
  
  Memberships = GetFuzzyMemberships(Samples, Centers, NormType, Blending);
  
  select convstr(Criterion, 'l')
  
   case 'iterations'
   
    if ~isdef('Iteration') then
     
     Iteration = 0;
     
    end;
    
    Iteration = Iteration + 1;
   
    IsCompleted = (Iteration > Value);
   
   case 'quantization_error'

    IsCompleted = (FuzzyQuantizationError(Samples, Centers, NormType) <= Value);
   
  else
  
   error('Unknown termination criterion. Criterion can be ''iterations''' + ...
         'or ''quantization_error''.');
      
  end;
  
  if ~IsCompleted then
   
   for n = 1 : NumberOfClusters

    Weight = repmat(Memberships(:, n) ^ Blending, 1, size(Samples, 2)) ;
                     
    Centers(n, :) = sum(Weight .* Samples, 'r') ./ sum(Weight, 'r');

   end;
   
  end;
  
 end;
  
endfunction